All files / packages/value/src type.ts

100% Statements 12/12
100% Branches 0/0
100% Functions 1/1
100% Lines 12/12

Press n or j to go to the next uncovered block, b, p or k for the previous block.

1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21    47x 47x 47x 47x 47x 47x 47x 47x 47x 47x 47x 47x              
import * as Ydb from '@ydbjs/api/value'
 
export enum TypeKind {
	PRIMITIVE = 1,
	DECIMAL = 2,
	OPTIONAL = 3,
	LIST = 4,
	TUPLE = 5,
	STRUCT = 6,
	DICT = 7,
	VARIANT = 8,
	VOID = 9,
	NULL = 10,
	PG_TYPE = 11,
}
 
export interface Type {
	kind: TypeKind
	encode(): Ydb.Type
}